/*
  mainwindow.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "mainwindow.h"

#include <qwidget.h>
#include <stdio.h>
#include "qgmap.h"
#include "qgconf.h"

#include <qdialog.h>
#include <qdatetime.h>

#include "pos.h"

#include "gmap.h"
#include "centerpos.h"
#include "slider.h"
#include "marker.h"
#include "iconmenu.h"
#include "scale.h"
#include "centermarker.h"
#include "freehandregion.h"

#include "help.h"
#include "info.h"

#ifndef QWS
#include <qlabel.h>
#endif

MainWindow::MainWindow( QWidget *parent, const char *name, WFlags fl) :
	LayeredBase(parent, name, fl)
{
#ifndef QWS
	resize(640,640);
#endif
	setCaption(QString::fromUtf8("qgmap"));

	gmap = new GMap(this,"gmap");
	center = new CenterPos(this,"center");
	slider = new Slider(this,"slider");
	marker = new Marker(this,"marker");
	menu = new IconMenu(this,"menu");
	scale = new Scale(this,"scale");
	centermarker = new CenterMarker(this,"centermarker");
	freehand = new FreehandRegion(this,"freehand");

	help = new Help(this);
	info = new Info(this);

	/* slider event */
	connect( slider, SIGNAL(valueChanged(int)),
			 center, SLOT(setZoom(int)));
	/* scroll event */
	connect( center, SIGNAL(positionChanged(Pos &)),
			 slider, SLOT(setPosition(Pos &)));
	connect( center, SIGNAL(positionChanged(Pos &)),
			 marker, SLOT(setPosition(Pos &)));
	connect( center, SIGNAL(positionChanged(Pos &)),
			 gmap, SLOT(setPosition(Pos &)));
	connect( center, SIGNAL(positionChanged(Pos &)),
			 scale, SLOT(setPosition(Pos &)));
	connect( center, SIGNAL(positionChanged(Pos &)),
			 freehand, SLOT(setPosition(Pos &)));

	/* config event */
	connect( &conf, SIGNAL(configChanged(const QString &, const QString &)),
			 gmap, SLOT(setConfig(const QString &, const QString &)));
	connect( &conf, SIGNAL(configChanged(const QString &, const QString &)),
			 marker, SLOT(setConfig(const QString &, const QString &)));

	addWidget(gmap);
	addWidget(center);
	addWidget(slider);
	addWidget(marker);
	addWidget(menu);
	addWidget(scale);
	addWidget(centermarker);
	addWidget(freehand);

#ifndef QWS
	label = new QLabel(this);
	label->setGeometry(640-16,640-16,16,16);
#endif
}

MainWindow::~MainWindow()
{
	delete gmap;
	delete slider;
	delete marker;
	delete menu;
	delete scale;
	delete centermarker;
	delete center;
}

void MainWindow::dispInfo(QString str)
{
	info->dispInfo(str);
}

void MainWindow::hideInfo()
{
	info->hide();
}

void MainWindow::mousePressEvent( QMouseEvent *e )
{
	if(help->isVisible()){
		help->hide();
		return;
	}

	LayeredBase::mousePressEvent(e);
}

void MainWindow::keyPressEvent ( QKeyEvent * e )
{
	if(help->isVisible()){
		help->hide();
		return;
	}

	switch(e->key()){
	case Key_H:
#ifndef QWS // for PC
	case Key_Question:
#endif
		help->show();
		return;

	case Key_Period:
		if(e->state() & ShiftButton){
			help->show();
			return;
		}
		break;
	}

	LayeredBase::keyPressEvent(e);
}

QString MainWindow::getConfig(const QString & key,
							   const QString & deflt) const
{
	return conf.readEntry(key, deflt);
}

int MainWindow::getNumConfig(const QString & key,
							 int deflt) const
{
	return conf.readNumEntry(key, deflt);
}

void MainWindow::setConfig(const QString & key, const QString & value)
{
	conf.writeEntry(key, value);
}

void MainWindow::setConfig(const QString & key, int value)
{
	conf.writeEntry(key, value);
}

void MainWindow::showEvent( QShowEvent *e )
{
	LayeredBase::showEvent(e);


	conf.reloadEntry("MapPath", DEFAULT_MAP_PATH);
	center->loadPosition(0);
}
