/*
  freehandregion.cpp
  Copyright (C) 2008 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "regionmap.h"
#include <stdio.h>

RegionMap::RegionMap(int left, int top, int width, int height, int zoom)
{
	this->top = top;
	this->left = left;
	this->width = width;
	this->height = height;
	this->zoom = zoom;
	this->map = new unsigned char[width*height];
	memset(this->map, 0, width*height);
}

RegionMap::RegionMap(int left, int top, int width, int height, int zoom, const unsigned char *map)
{
	this->top = top;
	this->left = left;
	this->width = width;
	this->height = height;
	this->zoom = zoom;
	this->map = new unsigned char[width*height];
	memcpy(this->map, map, width*height);
}

RegionMap::RegionMap(const RegionMap &pos)
{
	top = pos.top;
	left = pos.left;
	width = pos.width;
	height = pos.height;
	zoom = pos.zoom;
	map = new unsigned char[width*height];
	memcpy(map, pos.map, width*height);
}


RegionMap::~RegionMap()
{
	delete map;
}

RegionMap RegionMap::zoomTo(int to_zoom)
{
	int new_width, new_height;
	int new_left, new_top;

	if(zoom == to_zoom){
		return RegionMap(left, top, width, height, zoom, map);
	}

	if(zoom < to_zoom){
		// shrink
		int shift = to_zoom-zoom;
		new_left = left >> shift;
		new_top = top >> shift;
		new_width = width >> shift;
		new_height = height >> shift;
		if(new_width == 0)
			new_width = 1;
		if(new_height == 0)
			new_height = 1;

		RegionMap new_map(new_left, new_top, new_width, new_height, to_zoom);
		int x,y;

		for(y=0; y<height; y++){
			for(x=0; x<width; x++){
				if(map[y*width+x])
					new_map.map[(y>>shift)*new_width+(x>>shift)] = 0xff;
			}
		}
#ifdef DEBUG
		new_map.dump();
#endif
		return new_map;
	}

	// expand
	int shift = zoom-to_zoom;
	new_left = left << shift;
	new_top = top << shift;
	new_width = width << shift;
	new_height = height << shift;

	RegionMap new_map(new_left, new_top, new_width, new_height, to_zoom);
	int x,y;

	for(y=0; y<new_height; y++){
		for(x=0; x<new_width; x++){
			if(map[(y>>shift)*width+(x>>shift)])
				new_map.map[y*new_width+x] = 0xff;
		}
	}
	return new_map;
}

QArray<QRect> RegionMap::getRects()
{
	int x,y;
	int w,h;
	unsigned char *p = map;
	QArray<QRect> rarray(0);
	int i;

	mapSaturate();

	for(y = 0; y < height; y++){
		for(x = 0; x < width; x++){
			if(p[x]&1){
				w = scanWidth(x, y);
				h = scanHeight(x, y, w);
				i = rarray.size();
				rarray.resize(i+1);
				rarray[i] = QRect(x + left, y + top, w, h);
			}
		}
		p+=width;
	}
	return rarray;
}

int RegionMap::scanWidth(int x, int y)
{
	int i;
	unsigned char *p = map + y * width;

	for(i = x; i < width; i++){
		if(!(p[i]&1))
			break;
	}
	return i-x;
}

int RegionMap::scanHeight(int x, int y, int w)
{
	int i;
	int yy;
	unsigned char *p = map + y * width;
	for(yy=y+1; yy<height; yy++){
		for(i = x; i < x+w; i++)
			if(!(p[i]&1))
				break;
		if(i < x+w)
			break;

		memset(p+x,0xfe,w);
		p+=width;
	}
	return yy-y;
}

void RegionMap::mapSaturate()
{
	unsigned char *p = map;
	int size = width * height;

	while(size-->0){
		*p = *p ? 255 : 0;
		p++;
	}
}

RegionMap &RegionMap::operator=(const RegionMap &pos)
{
	if(this == &pos)
		return *this;

	width = pos.width;
	height = pos.height;
	zoom = pos.zoom;
	delete this->map;
	this->map = new unsigned char[width*height];
	memcpy(this->map, pos.map, width*height);
	return *this;
}

#ifdef DEBUG
void RegionMap::dump()
{
	printf("zoom=%d, (%d,%d)-(%d,%d)\n", zoom, left, top, left+width-1, top+height-1);
	unsigned char *p = map;
	int x,y;
	for(y=0;y<height;y++){
		for(x=0;x<width;x++){
			if(*p++)
				printf("8");
			else
				printf(".");
		}
		printf("\n");
	}
}
#endif
